@@ -13,8 +13,14 @@ from .models import WebSocketEventMessage
 
 class Handler:
     """
-    Class representation of a handler. It holds information about the handler, e.g. input model
-    (based on :class:`pydantic.BaseModel`), :param:`event`, etc
+    Class representation of a handler. It holds information about the handler, e.g. :attr:`model`
+    (based on :class:`pydantic.BaseModel`), :attr:`event`, etc
+
+    :param str event: name of the event this handler is for
+    :param typing.Callable method: method this handler will call
+    :param pydantic.BaseModel response_model:
+        :meth:`handle` will parse the return value of :attr:`method` into this model. If no model
+        is given a default response model will be created.
     """
 
     def __init__(
@@ -29,6 +35,8 @@ class Handler:
         self.bound_method: typing.Callable | None = None
 
         # create EventMessage model for input validation
+        #: Based on :class:`.WebSocketEventMessage` with fields for the parameters of
+        #: :attr:`method`. Will be used for input validation.
         self.model = create_model(
             f"WebSocketEventMessage_{self.event}",
             type=(typing.Literal[self.event], ...),
@@ -72,6 +80,13 @@ class Handler:
         return await method(*args, **kwargs)
 
     async def handle(self, msg: WebSocketEventMessage) -> BaseModel | None:
+        """
+        Will be called by :meth:`.WebSocketHandlingEndpoint.on_receive`
+
+        :param WebSocketEventMessage msg: will be validated against :attr:`model`
+        :returns: :attr:`response_model`
+        :rtype: WebSocketEventMessage
+        """
         errors = []
         field = self.response_field
         data = self.model.parse_obj(msg).dict(exclude={"type"})
@@ -95,19 +110,56 @@ class Handler:
         return value
 
 
+@typing.overload
+def on_event(event: typing.Callable) -> typing.Callable:
+    """
+    If the decorator is used without parentheses it's only argument will be the method itself.
+    :meta private:
+    """
+
+
+@typing.overload
 def on_event(
-    event: str, response_model: typing.Type[BaseModel] | None = None
+    event: str | None = None,
+    response_model: typing.Type[BaseModel] | None = None,
+) -> typing.Callable:
+    pass
+
+
+def on_event(
+    event: str | typing.Callable | None = None,
+    response_model: typing.Type[BaseModel] | None = None,
 ) -> typing.Callable:
     """
-    Should only be used in subclasses of :class:`WebSocketHandlingEndpoint`
-    Declares a method as handler for :param:`event`
+    Turns a :class:`typing.Callable` into a :class:`Handler`
+    Decorator to be used in subclasses of :class:`.WebSocketHandlingEndpoint`
+    Declares a method as handler for :attr:`event`
+
+    Takes the same parameters as :class:`Handler`
 
     Technical Note: Since it's impossible to get the class of an unbound function this decorator
     just sets some attributes on the function. The registration as handler happens in
-    :meth:`HandlingEndpointMeta.__new__`
+    :class:`.HandlingEndpointMeta`:meth:`.__new__`
     """
 
     def decorator(func: typing.Callable) -> Handler:
-        return Handler(event, func, response_model=response_model)
+        # if decorator is used without parantheses the first argument will be the function itself
+        event_name = None if callable(event) else event
+        if event_name is None:
+            if func.__name__.startswith("on_"):
+                event_name = func.__name__[3:]
+            elif func.__name__.startswith("handle_"):
+                event_name = func.__name__[7:]
+            else:
+                event_name = func.__name__
+            assert (
+                event_name is not None
+            ), "no event given and function doesn't start with on_ or handle_"
+        assert len(event_name) > 0, "event name has to be at leas 1 character"
 
-    return decorator
+        return Handler(event_name, func, response_model=response_model)
+
+    if callable(event):
+        return decorator(event)
+    else:
+        return decorator
